import json
from typing import TYPE_CHECKING, Callable, Dict, List, Optional, Union
from datetime import datetime

from AsyncYoomoney import (
    Account,
    History,
    OperationDetails,
)


class Client:
    def __init__(self,
                 token: str = None,
                 base_url: str = None,
                 ):

        if base_url is None:
            self.base_url = "https://yoomoney.ru/api/"

        if token is not None:
            self.token = token



    async def account_info(self):
        method = "account-info"
        return await Account.create(base_url=self.base_url,
                                    token=self.token,
                                    method=method
                                    )

    async def operation_history(self,
                          type: str = None,
                          label: str = None,
                          from_date: datetime = None,
                          till_date: datetime = None,
                          start_record: str = None,
                          records: int = None,
                          details: bool = None,
                          ):
        method = "operation-history"
        return await History.create(base_url=self.base_url,
                                    token=self.token,
                                    method=method,
                                    type=type,
                                    label=label,
                                    from_date=from_date,
                                    till_date=till_date,
                                    start_record=start_record,
                                    records=records,
                                    details=details,
                                    )

    async def operation_details(self,
                          operation_id: str
                          ):
        method = "operation-details"
        return await OperationDetails.create(base_url=self.base_url,
                                       token=self.token,
                                       method=method,
                                       operation_id=operation_id,
                                       )


from typing import List
import aiohttp

from AsyncYoomoney.exceptions import (
    InvalidRequest,
    UnauthorizedClient,
    InvalidGrant,
    EmptyToken
)

class Authorize:
    @classmethod
    async def create(
            cls,
            client_id: str,
            redirect_uri: str,
            client_secret: str,
            scope: List[str],
            session: aiohttp.ClientSession = None
                  ):

        url = "https://yoomoney.ru/oauth/authorize?client_id={client_id}&response_type=code" \
              "&redirect_uri={redirect_uri}&scope={scope}".format(client_id=client_id,
                                                                  redirect_uri=redirect_uri,
                                                                  scope='%20'.join([str(elem) for elem in scope]),
                                                                  )

        headers = {
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        if session:
            async with session.post(url, headers=headers) as response:
                response = response
        else:
            async with aiohttp.ClientSession() as session:
                async with session.post(url, headers=headers) as response:
                    response = response

        if response.status == 200:
            print("Visit this website and confirm the application authorization request:")
            print(response.url)

        code = str(input("Enter redirected url (https://yourredirect_uri?code=XXXXXXXXXXXXX) or just code: "))
        try:
            code = code[code.index("code=") + 5:].replace(" ","")
        except:
            pass

        url = "https://yoomoney.ru/oauth/token?code={code}&client_id={client_id}&" \
              "grant_type=authorization_code&redirect_uri={redirect_uri}&client_secret={client_secret}".format(code=str(code), client_id=client_id, redirect_uri=redirect_uri, client_secret=client_secret )
        
        if session:
            async with session.post(url, headers=headers) as response:
                response = response
        else:
            async with aiohttp.ClientSession() as session:
                async with session.post(url, headers=headers) as response:
                    response = response

        response_json = await response.json()

        if "error" in response_json:
            error = response_json["error"]
            if error == "invalid_request":
                raise InvalidRequest()
            elif error == "unauthorized_client":
                raise UnauthorizedClient()
            elif error == "invalid_grant":
                raise InvalidGrant()

        if response_json['access_token'] == "":
            raise EmptyToken()

        print("Your access token:")
        print(response_json['access_token'])


from .account.account import Account
from .account.balance_details import BalanceDetails

from .operation.operation import Operation

from AsyncYoomoney.operation_details.operation_details import OperationDetails
from AsyncYoomoney.operation_details.digital_bonus import DigitalBonus
from AsyncYoomoney.operation_details.digital_product import DigitalProduct
from AsyncYoomoney.operation_details.digital_good import DigitalGood

from .history.history import History

from .authorize.authorize import Authorize

from .quickpay.quickpay import Quickpay

from .client import Client

__all__ = [
    'Client',
    'Account',
    'BalanceDetails',
    'Operation',
    'History',
    'Authorize',
    "OperationDetails",
    "DigitalBonus",
    "DigitalProduct",
    "DigitalGood",
    "Quickpay",
    ]


import aiohttp
import json
from typing import TYPE_CHECKING, List

from AsyncYoomoney.account.balance_details import BalanceDetails
from AsyncYoomoney.account.card import Card
from AsyncYoomoney.exceptions import InvalidToken

class Account:

    @classmethod
    async def create(cls,
                 base_url: str = None,
                 token: str = None,
                 method: str = None,
                 session: aiohttp.ClientSession = None
                 ):
        self = cls()
        self.__private_method = method

        self.__private_base_url = base_url
        self.__private_token = token

        self.session = session

        data = await self._request()

        if len(data) != 0:
            self.account = data['account']
            self.balance = data['balance']
            self.currency = data['currency']
            self.account_status = data['account_status']
            self.account_type = data['account_type']

            self.balance_details = BalanceDetails()
            if 'balance_details' in data:
                if 'available' in data['balance_details']:
                    self.balance_details.available = float(data['balance_details']['available'])
                if 'blocked' in data['balance_details']:
                    self.balance_details.blocked = float(data['balance_details']['blocked'])
                if 'debt' in data['balance_details']:
                    self.balance_details.debt = float(data['balance_details']['debt'])
                if 'deposition_pending' in data['balance_details']:
                    self.balance_details.deposition_pending = float(data['balance_details']['deposition_pending'])
                if 'total' in data['balance_details']:
                    self.balance_details.total = float(data['balance_details']['total'])
                if 'hold' in data['balance_details']:
                    self.balance_details.hold = float(data['balance_details']['hold'])

            self.cards_linked = []
            if 'cards_linked' in data:
                for card_linked in data['cards_linked']:
                    card = Card(pan_fragment=card_linked['pan_fragment'], type=card_linked['type'])
                    self.cards_linked.append(card)
        else:
            raise InvalidToken()

        return self

    async def _request(self) -> aiohttp.ClientResponse:

        access_token = str(self.__private_token)
        url = self.__private_base_url + self.__private_method

        headers = {
            'Authorization': 'Bearer ' + str(access_token),
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        if self.session:
            async with self.session.post(url, headers=headers) as response:
                return await response.json()
            
        async with aiohttp.ClientSession() as session:
            async with session.post(url, headers=headers) as response:
                return await response.json()


import aiohttp

class Quickpay:
    @classmethod
    async def create(cls,
                 receiver: str,
                 quickpay_form : str,
                 targets: str,
                 paymentType: str,
                 sum: float,
                 formcomment: str = None,
                 short_dest: str = None,
                 label: str = None,
                 comment: str = None,
                 successURL: str = None,
                 need_fio: bool = None,
                 need_email: bool = None,
                 need_phone: bool = None,
                 need_address: bool = None,
                 session: aiohttp.ClientSession = None
                 ):
        self = cls()
        self.receiver = receiver
        self.quickpay_form = quickpay_form
        self.targets = targets
        self.paymentType = paymentType
        self.sum = sum
        self.formcomment = formcomment
        self.short_dest = short_dest
        self.label = label
        self.comment = comment
        self.successURL = successURL
        self.need_fio = need_fio
        self.need_email = need_email
        self.need_phone = need_phone
        self.need_address = need_address
        self.session = session
        self.response = await self._request()

        return self

    async def _request(self):
        self.base_url = "https://yoomoney.ru/quickpay/confirm.xml?"
        payload = {}

        payload["receiver"] = self.receiver
        payload["quickpay_form"] = self.quickpay_form
        payload["targets"] = self.targets
        payload["paymentType"] = self.paymentType
        payload["sum"] = self.sum

        if self.formcomment != None:
            payload["formcomment"] = self.formcomment
        if self.short_dest != None:
            payload["short_dest"] = self.short_dest
        if self.label != None:
            payload["label"] = self.label
        if self.comment != None:
            payload["comment"] = self.comment
        if self.successURL != None:
            payload["successURL"] = self.successURL
        if self.need_fio != None:
            payload["need_fio"] = self.need_fio
        if self.need_email != None:
            payload["need_email"] = self.need_email
        if self.need_phone != None:
            payload["need_phone"] = self.need_phone
        if self.need_address != None:
            payload["need_address"] = self.need_address

        for value in payload:
            self.base_url+=str(value).replace("_","-") + "=" + str(payload[value])
            self.base_url+="&"

        self.base_url = self.base_url[:-1].replace(" ", "%20")

        if self.session:
            async with self.session.get(self.base_url) as response:
                response = response
        else:
            async with aiohttp.ClientSession() as session:
                async with session.get(self.base_url) as response:
                    response = response

        self.redirected_url = response.url
        return response


from setuptools import setup, find_packages, Extension

setup(
    name='AsyncYoomoney',
    version='0.1.3',
    packages=find_packages(exclude=['tests']),
    url='https://github.com/vxjnc/async-yoomoney-api.git',
    license='GPL-3.0',
    author='vxjnc',
    author_email='vxjnc0@gmail.com',
    description='Unofficial YooMoney API python library',
    long_description=str(open('README.rst').read())
)
